// Copyright 2011 David Galles, University of San Francisco. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice, this list of
// conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice, this list
// of conditions and the following disclaimer in the documentation and/or other materials
// provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY David Galles ``AS IS'' AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of the University of San Francisco

var ctx;
var points = [];
var nextindex;
var computedIndex;
var deletedpoints = [];
function ConvexHull(am, w, h) {
    this.init(am, w, h);
}

ConvexHull.prototype = new Algorithm();
ConvexHull.prototype.constructor = ConvexHull;
ConvexHull.superclass = Algorithm.prototype;

ConvexHull.HIGHLIGHT_CIRCLE_COLOR = "#007700";

ConvexHull.FOREGROUND_COLOR = "#000F00"
ConvexHull.DRAWPOINT_COLOR = "#F63752"
ConvexHull.UPPERHULL_COLOR = "#1500BB"
ConvexHull.LOWERHULL_COLOR = "#0D8300"

ConvexHull.prototype.init = function (am, w, h) {
    // Call the unit function of our "superclass", which adds a couple of
    // listeners, and sets up the undo stack
    ConvexHull.superclass.init.call(this, am, w, h);

    this.addControls();

    // Useful for memory management
    this.commands = [];
    this.setup();
    this.convexList = [];

}

ConvexHull.prototype.setup = function () {
    computedIndex = 0;
    nextindex = 10;
    ctx = canvas.getContext('2d');
    canvas.addEventListener("click", function (event) {
        var elem = new point(event.pageX, event.pageY - 90);
        elem.create();
    }, false);

    document.addEventListener('contextmenu', function (e) {

        for (var i = 0; i < points.length; i++) {
            if ((points[i].x + 5 > e.pageX && points[i].x - 10 < e.pageX) && ((points[i].y + 5 > e.pageY - 90 && points[i].y - 10 < e.pageY - 90))) {
                ctx.clearRect(points[i].x - 5 - 1, points[i].y - 5 - 1, 5 * 2 + 2, 5 * 2 + 2);
                if (i < computedIndex) {
                    computedIndex--;
                    deletedpoints.push(points[i]);
                }
                points.splice(i, 1);
                break;
            }
        }
        e.preventDefault();
    }, false);
    this.cmd("CreateLabel", 0, "", 20, 10, 0);
    this.animationManager.StartNewAnimation(this.commands);
    this.animationManager.skipForward();
    this.animationManager.clearHistory();
}

ConvexHull.prototype.addControls = function () {
    this.controls = [];

    this.ha = addControlToAlgorithmBar("Button", "About ConvexHull");
    this.ha.onclick = this.haCallback.bind(this);

    addLabelToAlgorithmBar("X: ");
    this.xPointField = addControlToAlgorithmBar("Text", "", "<div id='xtooltip'>0 < X < " + canvas.width + "</div>");
    this.xPointField.onkeypress = this.returnSubmit(this.xPointField,        // callback to make when return is pressed
                                               this.insert.bind(this),           // integer, max number of characters allowed in field
                                               4,                               // boolean, true of only digits can be entered.
                                               true);
    this.controls.push(this.xPointField);

    addLabelToAlgorithmBar("Y: ");
    this.yPointField = addControlToAlgorithmBar("Text", "", "<div id='ytooltip'>0 < Y < " + canvas.height + "</div>");
    this.yPointField.onkeypress = this.returnSubmit(this.yPointField,
                                               this.insert.bind(this),
                                               4,
                                               true);
    this.controls.push(this.yPointField);

    this.insertButton = addControlToAlgorithmBar("Button", "Insert");
    this.insertButton.onclick = this.insert.bind(this);
    this.controls.push(this.insertButton);

    this.computeButton = addControlToAlgorithmBar("Button", "Compute");
    this.computeButton.onclick = this.computeCallback.bind(this);
    this.controls.push(this.computeButton);
}

ConvexHull.prototype.reset = function () {
    for (var i = 1; i < this.convexList.length; i++) {
        this.cmd("disconnect", this.convexList[i - 1].objectId, this.convexList[i].objectId);
    }
}
ConvexHull.prototype.insert = function () {
    var xValue = this.xPointField.value;
    var yValue = this.yPointField.value;
    if (xValue != "" && yValue != "") {
        var elem = new point(xValue, yValue);
        elem.create();
        this.xPointField.value = "";
        this.yPointField.value = "";
    }
}

ConvexHull.prototype.computeCallback = function () {
    this.implementAction(this.compute.bind(this), "");
}
ConvexHull.prototype.haCallback = function () {
    window.open("AboutAlgorithm/AboutConvexHull.html", "MsgWindow", "width=700, height=520");
}
function sortByXValue(a, b) {
    return ((a.x < b.x) ? -1 : (a.x > b.x) ? 1 : 0);
}
function isLeftturn(a, b, c) {
    return !(((b.x - a.x) * (c.y - a.y) - (b.y - a.y) * (c.x - a.x)) > 0);
}
ConvexHull.prototype.compute = function (pushedValue) {
    this.commands = [];
    this.reset();

    for (var l = 0; l < deletedpoints.length; l++) {
        this.cmd("delete", deletedpoints[l].objectId);
    }
    deletedpoints = [];

    for (var i = computedIndex; i < points.length; i++) {
        this.cmd("CREATEPOINT", points[i].objectId, points[i].x, points[i].y);
        this.cmd("SetBackgroundColor", points[i].objectId, ConvexHull.DRAWPOINT_COLOR);
    }
    points.sort(sortByXValue);
    this.cmd("Step");

    if (points.length > 1) {
        var lUpper = [];
        lUpper.push(points[0]);
        lUpper.push(points[1]);
        this.cmd("CreateHighlightCircle", 1, ConvexHull.HIGHLIGHT_CIRCLE_COLOR, lUpper[0].x, lUpper[0].y, 7);
        this.cmd("Connect", lUpper[0].objectId, lUpper[1].objectId, ConvexHull.UPPERHULL_COLOR, 0.0, 0);
        this.cmd("Step");
        for (var i = 2; i < points.length; i++) {
            lUpper.push(points[i]);
            this.makeConvexList(lUpper, ConvexHull.UPPERHULL_COLOR);
        }

        this.cmd("SetText", 0, "UpperHull is maked");
        this.cmd("Step");


        var lLower = [];
        lLower.push(points[points.length - 1]);
        lLower.push(points[points.length - 2]);
        this.cmd("Connect", lLower[0].objectId, lLower[1].objectId, ConvexHull.LOWERHULL_COLOR, 0.0, 0);
        this.cmd("Step");
        for (var i = points.length - 3; i >= 0; i--) {
            lLower.push(points[i]);
            this.makeConvexList(lLower, ConvexHull.LOWERHULL_COLOR);
        }

        this.cmd("Delete", 1);
        lLower.splice(0, 1);
        this.convexList = lUpper.concat(lLower);
        this.cmd("SetText", 0, "LowerHull is maked");
        this.cmd("Step");
        this.cmd("SetText", 0, "");

    }
    computedIndex = points.length;
    return this.commands;
}

ConvexHull.prototype.makeConvexList = function (list, color) {
    this.cmd("Move", 1, list[list.length - 1].x, list[list.length - 1].y);
    this.cmd("Step");
    this.cmd("Connect", list[list.length - 2].objectId, list[list.length - 1].objectId, color, 0.0, 0);
    this.cmd("Step");
    while (list.length > 2 && isLeftturn(list[list.length - 3], list[list.length - 2], list[list.length - 1])) {
        this.cmd("SetHighlight", list[list.length - 2].objectId, 1);
        this.cmd("SetHighlight", list[list.length - 3].objectId, 1);
        this.cmd("Step");
        this.cmd("disconnect", list[list.length - 2].objectId, list[list.length - 1].objectId);
        this.cmd("disconnect", list[list.length - 3].objectId, list[list.length - 2].objectId);
        this.cmd("SetText", 0, "It is not right turn");
        this.cmd("Connect", list[list.length - 3].objectId, list[list.length - 1].objectId, color, 0.0, 0);
        this.cmd("SetHighlight", list[list.length - 2].objectId, 0);
        this.cmd("SetHighlight", list[list.length - 3].objectId, 0);
        this.cmd("Step");
        list.splice(list.length - 2, 1);
    }
    this.cmd("SetText", 0, "");
}


function point(xValue, yValue) {
    this.x = xValue;
    this.y = yValue;
    this.objectId = nextindex;
    this.create = function () {
        ctx.beginPath();
        ctx.arc(this.x, this.y, 5, 0, 2 * Math.PI, false);
        ctx.fillStyle = "#000000";
        ctx.fill();
        ctx.closePath();
        points.push(this);
        nextindex++;
    }
}


// Called by our superclass when we get an animation started event -- need to wait for the
// event to finish before we start doing anything
ConvexHull.prototype.disableUI = function (event) {
    for (var i = 0; i < this.controls.length; i++) {
        this.controls[i].disabled = true;
    }
}

// Called by our superclass when we get an animation completed event -- we can
/// now interact again.
ConvexHull.prototype.enableUI = function (event) {
    for (var i = 0; i < this.controls.length; i++) {
        this.controls[i].disabled = false;
    }
}

////////////////////////////////////////////////////////////
// Script to start up your function, called from the webapge:
////////////////////////////////////////////////////////////

var currentAlg;

function init() {
    var animManag = initCanvas();
    currentAlg = new ConvexHull(animManag, canvas.width, canvas.height);

}


//function point(x1, y1) {
//    this.x = x1;
//    this.y = y1;
//}