﻿/**
 * Basic priority queue implementation. If a better priority queue is wanted/needed,
 * this code works with the implementation in google's closure library (https://code.google.com/p/closure-library/).
 * Use goog.require('goog.structs.PriorityQueue'); and new goog.structs.PriorityQueue()
 */
function PriorityQueue() {
    this._nodes = [];

    this.enqueue = function (priority, key) {
        this._nodes.push({ key: key, priority: priority });
        this.sort();
    }
    this.dequeue = function () {
        return this._nodes.shift().key;
    }
    this.sort = function () {
        this._nodes.sort(function (a, b) {
            return a.priority - b.priority;
        });
    }
    this.isEmpty = function () {
        return !this._nodes.length;
    }
}

/**
 * Pathfinding starts here
 */
function DijkstraGraph() {
    var INFINITY = 1 / 0;
    this.vertices = {};
    this.returnPath = [];
    this.addVertex = function (name, edges) {
        this.vertices[name] = edges;
    }

    this.addEdge = function (vertexName, targetVertex, weight) {
        if (this.vertices[vertexName] === undefined) {
            this.vertices[vertexName] = {};
        }
        if (this.vertices[targetVertex] === undefined) {
            this.vertices[targetVertex] = {};
        }
        this.vertices[vertexName][targetVertex] = weight;
        this.vertices[targetVertex][vertexName] = weight;

    }

    this.shortestPath = function (start, finish) {
        var nodes = new PriorityQueue(),
            distances = {},
            previous = {},
            path = [],
            sumWeight = 0,
            smallest, vertex, neighbor, alt;

        for (vertex in this.vertices) {
            if (vertex === start) {
                distances[vertex] = 0;
                nodes.enqueue(0, vertex);
            }
            else {
                distances[vertex] = INFINITY;
                nodes.enqueue(INFINITY, vertex);
            }

            previous[vertex] = null;
        }

        while (!nodes.isEmpty()) {
            smallest = nodes.dequeue();

            if (smallest === finish) {
                path;

                while (previous[smallest]) {
                    path.push(smallest);
                    smallest = previous[smallest];
                }

                break;
            }

            if (!smallest || distances[smallest] === INFINITY) {
                continue;
            }

            for (neighbor in this.vertices[smallest]) {
                alt = distances[smallest] + this.vertices[smallest][neighbor];

                if (alt < distances[neighbor]) {
                    distances[neighbor] = alt;
                    previous[neighbor] = smallest;

                    nodes.enqueue(alt, neighbor);
                }
            }
        }
        path.push(start);
        for (var i = 0; i < path.length - 1; i++) {
            sumWeight += this.vertices[path[i]][path[i + 1]];
        }
        this.returnPath = path;
        return (sumWeight == 0) ? Infinity : sumWeight;
    }
}