// Copyright 2011 David Galles, University of San Francisco. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice, this list of
// conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice, this list
// of conditions and the following disclaimer in the documentation and/or other materials
// provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY David Galles ``AS IS'' AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of the University of San Francisco

var ctx;
var points = [];
var nextindex;
var theta, w;
var computedIndex;
var deletedpoints = [];
function GapGreedy(am, w, h) {
    this.init(am, w, h);
}

GapGreedy.prototype = new Algorithm();
GapGreedy.prototype.constructor = GapGreedy;
GapGreedy.superclass = Algorithm.prototype;

GapGreedy.HIGHLIGHT_CIRCLE_COLOR = "#007700";
GapGreedy.HIGHLIGHT_CIRCLE_COLOR_LAST = "#953C00";

GapGreedy.TEXT_TRUE_COLOR = "#0C9200";
GapGreedy.TEXT_FALSE_COLOR = "#E50007";

GapGreedy.FOREGROUND_COLOR = "#000000"
GapGreedy.DRAWPOINT_COLOR = "#F63752"

GapGreedy.prototype.init = function (am, w, h) {
    // Call the unit function of our "superclass", which adds a couple of
    // listeners, and sets up the undo stack
    GapGreedy.superclass.init.call(this, am, w, h);

    this.addControls();

    // Useful for memory management
    this.commands = [];
    this.setup();
    this.greedyList = [];


}

GapGreedy.prototype.setup = function () {
    computedIndex = 0;
    nextindex = 10;
    ctx = canvas.getContext('2d');
    canvas.addEventListener("click", function (event) {
        var elem = new point(event.pageX, event.pageY - 90);
        elem.create();
    }, false);

    document.addEventListener('contextmenu', function (e) {

        for (var i = 0; i < points.length; i++) {
            if ((points[i].x + 5 > e.pageX && points[i].x - 10 < e.pageX) && ((points[i].y + 5 > e.pageY - 90 && points[i].y - 10 < e.pageY - 90))) {
                ctx.clearRect(points[i].x - 5 - 1, points[i].y - 5 - 1, 5 * 2 + 2, 5 * 2 + 2);
                if (i < computedIndex) {
                    computedIndex--;
                    deletedpoints.push(points[i]);
                }
                points.splice(i, 1);
                break;
            }
        }
        e.preventDefault();
    }, false);
    this.cmd("CreateLabel", 0, "", 20, 10, 0);
    this.animationManager.StartNewAnimation(this.commands);
    this.animationManager.skipForward();
    this.animationManager.clearHistory();

}

GapGreedy.prototype.addControls = function () {
    this.controls = [];

    this.ha = addControlToAlgorithmBar("Button", "About Gap greedy");
    this.ha.onclick = this.haCallback.bind(this);

    addLabelToAlgorithmBar("X: ");
    this.xPointField = addControlToAlgorithmBar("Text", "", "<div id='xtooltip'>0 < X < " + canvas.width + "</div>");
    this.xPointField.onkeypress = this.returnSubmit(this.xPointField,        // callback to make when return is pressed
                                               this.insert.bind(this),           // integer, max number of characters allowed in field
                                               4,                               // boolean, true of only digits can be entered.
                                               true);
    this.controls.push(this.xPointField);

    addLabelToAlgorithmBar("Y: ");
    this.yPointField = addControlToAlgorithmBar("Text", "", "<div id='ytooltip'>0 < Y < " + canvas.height + "</div>");
    this.yPointField.onkeypress = this.returnSubmit(this.yPointField,
                                               this.insert.bind(this),
                                               4,
                                               true);
    this.controls.push(this.yPointField);

    this.insertButton = addControlToAlgorithmBar("Button", "Insert");
    this.insertButton.onclick = this.insert.bind(this);
    this.controls.push(this.insertButton);

    addLabelToAlgorithmBar("θ: ");
    this.tValue = addControlToAlgorithmBar("Text", "", "Please insert degree θ such that 0 < θ < 45");
    this.tValue.onkeypress = this.returnSubmit(this.tValue,
                                               this.computeCallback.bind(this),
                                               8,
                                               true);
    this.controls.push(this.tValue);


    addLabelToAlgorithmBar("w: ");
    this.wValue = addControlToAlgorithmBar("Text", "0.", "<img src='img/GapGreedt-w.png'>");
    this.wValue.onkeypress = this.returnSubmit(this.wValue,
                                               this.computeCallback.bind(this),
                                               8,
                                               true);
    this.controls.push(this.wValue);

    this.computeButton = addControlToAlgorithmBar("Button", "Compute");
    this.computeButton.onclick = this.computeCallback.bind(this);
    this.controls.push(this.computeButton);
}

GapGreedy.prototype.reset = function () {
    for (var i = 0; i < this.greedyList.length; i++) {
        this.cmd("disconnect", this.greedyList[i].a.objectId, this.greedyList[i].b.objectId);
    }
    this.greedyList = [];
}
GapGreedy.prototype.insert = function () {
    var xValue = this.xPointField.value;
    var yValue = this.yPointField.value;
    if (xValue != "" && yValue != "") {
        var elem = new point(xValue, yValue);
        elem.create();
        this.xPointField.value = "";
        this.yPointField.value = "";
    }
}

GapGreedy.prototype.computeCallback = function () {
    w = parseFloat("0" + this.wValue.value);
    theta = parseFloat("0" + this.tValue.value);
    if (this.tValue.value.trim() == "") {
        alert("Please insert θ value");
        return;
    }
    if (theta <= 0 || theta >= 45) {
        alert("Please insert t value such that 0 < θ < 45");
        return;
    }
    var max_w = (Math.cos(Math.PI / (180 / theta)) - Math.sin(Math.PI / (180 / theta))) / 2;
    if (w <= 0 || w >= max_w) {
        alert("Please insert w value with correct format  between 0 and " + max_w);
        return;
    }
    this.implementAction(this.compute.bind(this), "");
}

GapGreedy.prototype.haCallback = function () {
    window.open("AboutAlgorithm/AboutGapGreedy.html", "MsgWindow", "width=750, height=650");
}

function sortDistance(a, b) {
    return ((a.dis < b.dis) ? -1 : (a.dis > b.dis) ? 1 : 0);
}

function computeAngle(_p0, _p1, _p2) {
    var angle1 = Math.atan2(_p0.y - _p1.y, _p0.x - _p1.x);
    var angle2 = Math.atan2(_p1.y - _p2.y, _p1.x - _p2.x);


    var angle = angle1 - angle2;
    angle = ((Math.PI + angle) / Math.PI * 180);
    if (angle > 180)
        angle = 360 - angle;
    if (angle == 0)
        angle = 180;


    return Math.abs(angle);
}

function computeDistanse(a, b) {
    return Math.sqrt(Math.pow((b.x - a.x), 2) + Math.pow((b.y - a.y), 2));

}
GapGreedy.prototype.compute = function (pushedValue) {
    this.commands = [];
    this.reset();
    for (var l = 0; l < deletedpoints.length; l++) {
        this.cmd("delete", deletedpoints[l].objectId);
    }
    deletedpoints = [];

    for (var i = computedIndex; i < points.length; i++) {
        this.cmd("CREATEPOINT", points[i].objectId, points[i].x, points[i].y);
        this.cmd("SetBackgroundColor", points[i].objectId, GapGreedy.DRAWPOINT_COLOR);
    }
    this.cmd("Step");

    var distances = [];
    for (var i = 0; i < points.length; i++) {
        for (var j = 0; j < points.length; j++) {
            if (i !== j) {
                var dist = new distance(points[i], points[j]);
                distances.push(dist);
            }
        }
    }
    distances.sort(sortDistance);
    this.cmd("CreateHighlightCircle", 1, GapGreedy.HIGHLIGHT_CIRCLE_COLOR, distances[0].a.x, distances[0].a.y, 7);
    this.cmd("CreateHighlightCircle", 2, GapGreedy.HIGHLIGHT_CIRCLE_COLOR_LAST, distances[0].b.x, distances[0].b.y, 7);
    this.cmd("CREATEPOINT", 3, -10, -10);
    this.cmd("CREATEPOINT", 4, -10, -10);
    this.cmd("Connect", 3, 4, "#AAAAAA", 0.0, 0);



    for (i in distances) {
        var dist = distances[i];

        var add = true;
        this.cmd("Move", 1, dist.a.x, dist.a.y);
        this.cmd("Move", 2, dist.b.x, dist.b.y);
        this.cmd("Connect", dist.a.objectId, dist.b.objectId, "#FFFF00", 0.0, 0);
        this.cmd("Step");
        var p = dist.a;
        var q = dist.b;
        for (var j = 0; j < this.greedyList.length; j++) {

            var r = this.greedyList[j].a;
            var s = this.greedyList[j].b;
            this.cmd("SetEdgeHighlight", r.objectId, s.objectId, 1);
            this.cmd("SetEdgeHighlight", s.objectId, r.objectId, 1);

            var s_translated = new point(this.greedyList[j].b.x + dist.a.x - this.greedyList[j].a.x,
                this.greedyList[j].b.y + dist.a.y - this.greedyList[j].a.y);
            //this.cmd("Move", 3, r.x, r.y);
            //this.cmd("Move", 4, s.x, s.y);
            //this.cmd("step");
            //this.cmd("step");

            this.cmd("Move", 3, p.x, p.y);
            this.cmd("Move", 4, s_translated.x, s_translated.y);
            this.cmd("step");

            angle_pqrs = computeAngle(q, p, s_translated);
            var text = "angle(pq,rs) = " + angle_pqrs.toFixed(3) + "  <  " + theta + "= " + (angle_pqrs < theta);
            this.cmd("SetText", 0, text);
            if (angle_pqrs < theta) {
                this.cmd("SetTextColor", 0, GapGreedy.TEXT_TRUE_COLOR);
                var statement1 = add;
                var statement2 = computeDistanse(p, r) > w * computeDistanse(r, s);
                var statement3 = computeDistanse(q, s) > w * computeDistanse(r, s);
                add = statement1 && statement2 && statement3;
                text += "\n" + "add= " + statement1 + " && " + statement2 + " && " + statement3 + " = " + add;
                this.cmd("SetText", 0, text);

                this.cmd("Step");
            }
            else {
                this.cmd("SetTextColor", 0, GapGreedy.TEXT_FALSE_COLOR);
                this.cmd("Step");
            }
            this.cmd("SetEdgeHighlight", r.objectId, s.objectId, 0);
            this.cmd("SetEdgeHighlight", s.objectId, r.objectId, 0);



        }
        this.cmd("disConnect", dist.a.objectId, dist.b.objectId, GapGreedy.FOREGROUND_COLOR, 0.0, 0);
        if (add) {
            this.greedyList.push(dist);
            this.cmd("Connect", dist.a.objectId, dist.b.objectId, GapGreedy.FOREGROUND_COLOR, 0.0, 0);
            this.cmd("Step");
        }
    }
    this.cmd("SetText", 0, "");
    this.cmd("Delete", 1);
    this.cmd("Delete", 2);
    this.cmd("Delete", 3);
    this.cmd("Delete", 4);


    computedIndex = points.length;
    return this.commands;
}



function point(xValue, yValue) {
    this.x = xValue;
    this.y = yValue;
    this.objectId = nextindex;
    this.create = function () {
        ctx.beginPath();
        ctx.arc(this.x, this.y, 5, 0, 2 * Math.PI, false);
        ctx.fillStyle = "#000000";
        ctx.fill();
        ctx.closePath();
        points.push(this);
        nextindex++;
    }
}

function distance(a, b) {
    this.a = a;
    this.b = b;
    this.dis = Math.sqrt(Math.pow((b.x - a.x), 2) + Math.pow((b.y - a.y), 2));
}


// Called by our superclass when we get an animation started event -- need to wait for the
// event to finish before we start doing anything
GapGreedy.prototype.disableUI = function (event) {
    for (var i = 0; i < this.controls.length; i++) {
        this.controls[i].disabled = true;
    }
}

// Called by our superclass when we get an animation completed event -- we can
/// now interact again.
GapGreedy.prototype.enableUI = function (event) {
    for (var i = 0; i < this.controls.length; i++) {
        this.controls[i].disabled = false;
    }
}

////////////////////////////////////////////////////////////
// Script to start up your function, called from the webapge:
////////////////////////////////////////////////////////////

var currentAlg;

function init() {
    var animManag = initCanvas();
    currentAlg = new GapGreedy(animManag, canvas.width, canvas.height);

}


//function point(x1, y1) {
//    this.x = x1;
//    this.y = y1;
//}