// Copyright 2011 David Galles, University of San Francisco. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice, this list of
// conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice, this list
// of conditions and the following disclaimer in the documentation and/or other materials
// provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY <COPYRIGHT HOLDER> ``AS IS'' AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of the University of San Francisco


var AnimatedPoint = function (objectID, _radious) {
    this.objectID = objectID;
    this.radius = _radious;
    this.thickness = 3;
    this.x = 0;
    this.y = 0;
    this.alpha = 1.0;
    this.addedToScene = true;
    /*	this.foregroundColor  = '#007700';
        this.backgroundColor  = '#EEFFEE';
     */
}

AnimatedPoint.prototype = new AnimatedObject();
AnimatedPoint.prototype.constructor = AnimatedPoint;

AnimatedPoint.prototype.getTailPointerAttachPos = function (fromX, fromY, anchorPoint) {
    return this.getHeadPointerAttachPos(fromX, fromY);
}


AnimatedPoint.prototype.getWidth = function () {
    return this.radius * 2;
}


AnimatedPoint.prototype.getHeadPointerAttachPos = function (fromX, fromY) {
    var xVec = fromX - this.x;
    var yVec = fromY - this.y;
    var len = Math.sqrt(xVec * xVec + yVec * yVec);
    if (len == 0) {
        return [this.x, this.y];
    }
    return [this.x + (xVec / len) * (this.radius), this.y + (yVec / len) * (this.radius)];
}

AnimatedPoint.prototype.draw = function (ctx) {
    ctx.globalAlpha = this.alpha;

    if (this.highlighted) {
        ctx.fillStyle = "#ff0000";
        ctx.beginPath();
        ctx.arc(this.x, this.y, this.radius + this.highlightDiff, 0, Math.PI * 2, true);
        ctx.closePath();
        ctx.fill();
    }

    ctx.fillStyle = this.backgroundColor;
    ctx.strokeStyle = this.foregroundColor;
    ctx.lineWidth = 1;
    ctx.beginPath();
    ctx.arc(this.x, this.y, this.radius, 0, Math.PI * 2, true);
    ctx.closePath();
    ctx.fill();
    ctx.stroke();
    ctx.textAlign = 'center';
    ctx.font = '10px sans-serif';
    ctx.textBaseline = 'middle';
    ctx.lineWidth = 1;
    ctx.fillStyle = this.foregroundColor;

}


AnimatedPoint.prototype.createUndoDelete = function () {
    return new UndoDeletePoint(this.objectID, this.x, this.y, this.foregroundColor, this.backgroundColor, this.layer);
}


function UndoDeletePoint(id, x, y, foregroundColor, backgroundColor, l) {
    this.objectID = id;
    this.posX = x;
    this.posY = y;
    this.fgColor = foregroundColor;
    this.bgColor = backgroundColor;
    this.layer = l;
}

UndoDeletePoint.prototype = new UndoBlock();
UndoDeletePoint.prototype.constructor = UndoDeletePoint;

UndoDeletePoint.prototype.undoInitialStep = function (world) {
    world.addPointObject(this.objectID, 5);
    world.setNodePosition(this.objectID, this.posX, this.posY);
    world.setForegroundColor(this.objectID, this.fgColor);
    world.setBackgroundColor(this.objectID, this.bgColor);
    world.setLayer(this.objectID, this.layer);
}




