// Copyright 2011 David Galles, University of San Francisco. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice, this list of
// conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice, this list
// of conditions and the following disclaimer in the documentation and/or other materials
// provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY <COPYRIGHT HOLDER> ``AS IS'' AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of the University of San Francisco




var LINE_maxHeightDiff = 5;
var LINE_minHeightDiff = 3;
var LINE_range = LINE_maxHeightDiff - LINE_minHeightDiff + 1;
var LINE_highlightDiff = 3;


function CustomLine(objectID, x, y, x2, y2, color) {
    this.arrowHeight = 8;
    this.arrowWidth = 4;

    this.objectID = objectID;
    this.x = x;
    this.x2 = x2;
    this.y = y;
    this.y2 = y2;
    this.Dirty = false;
    this.edgeColor = color;
    this.highlighted = false;
    this.addedToScene = true;
    this.highlightDiff = 0;

    this.alpha = 1.0;

    this.color = function color() {
        return this.edgeColor;
    }

    this.setColor = function (newColor) {
        this.edgeColor = newColor;
        Dirty = true;
    }

    this.setHighlight = function (highlightVal) {
        this.highlighted = highlightVal;
    }

    this.pulseHighlight = function (frameNum) {
        if (this.highlighted) {
            var frameMod = frameNum / 14.0;
            var delta = Math.abs((frameMod) % (2 * LINE_range - 2) - LINE_range + 1)
            this.highlightDiff = delta + LINE_minHeightDiff;
            Dirty = true;
        }
    }


    //this.hasNode = function (n) {
    //    return ((this.Node1 == n) || (this.Node2 == n));
    //}


    this.createUndoLine = function () {
        return new UndoLine(this.objectID, this.x, this.y, this.x2, this.y2, true, this.edgeColor);
    }


    this.sign = function (n) {
        if (n > 0) {
            return 1;
        }
        else {
            return -1;
        }
    }


    this.drawArrow = function (pensize, color, context) {
        context.strokeStyle = color;
        context.fillStyle = color;
        context.lineWidth = pensize;

        context.beginPath();
        context.moveTo(this.x, this.y);
        context.lineTo(this.x2, this.y2);
        context.stroke();


    }


    this.draw = function (ctx) {
        if (!this.addedToScene) {
            return;
        }
        ctx.globalAlpha = this.alpha;

        if (this.highlighted)
            this.drawArrow(this.highlightDiff, "#FF0000", ctx);
        this.drawArrow(1, this.edgeColor, ctx);
    }


}



function UndoLine(Id, fromX, fromY, toX, toY, createConnection, lineColor) {
    this.objectID = id;
    this.fromX = fromX;
    this.fromY = fromY;
    this.toX = toX;
    this.toY = toY;
    this.connect = createConnection;
    this.color = lineColor;

}


UndoLine.prototype.undoInitialStep = function (world) {
    if (this.connect) {
        world.createLine(this.objectID, this.fromX, this.fromY, this.toX, this.toY, this.color);
    }
    else {
        world.removeObject(this.objectID);
    }
}


UndoLine.prototype.addUndoAnimation = function (animationList) {
    return false;
}
