// Copyright 2011 David Galles, University of San Francisco. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice, this list of
// conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice, this list
// of conditions and the following disclaimer in the documentation and/or other materials
// provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY David Galles ``AS IS'' AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of the University of San Francisco

var ctx;
var points = [];
var nextindex;
var nextindexExplorer;
var computedIndex;
var deletedpoints = [];

function ThetaGraph(am, w, h) {
    this.init(am, w, h);
}

ThetaGraph.prototype = new Algorithm();
ThetaGraph.prototype.constructor = ThetaGraph;
ThetaGraph.superclass = Algorithm.prototype;

ThetaGraph.HIGHLIGHT_CIRCLE_COLOR = "#007700";
ThetaGraph.PROJECT_COLOR = "#F63752";
ThetaGraph.FOREGROUND_COLOR = "#000F00";
ThetaGraph.DRAWPOINT_COLOR = "#F63752"
ThetaGraph.EPLORER_RADIOUS = 1000;
ThetaGraph.STARTPOINT_X = 30;
ThetaGraph.STARTPOINT_Y = 30;

ThetaGraph.prototype.init = function (am, w, h) {
    // Call the unit function of our "superclass", which adds a couple of
    // listeners, and sets up the undo stack
    ThetaGraph.superclass.init.call(this, am, w, h);

    this.addControls();

    // Useful for memory management
    this.commands = [];
    this.setup();
    this.convexList = [];


}

ThetaGraph.prototype.setup = function () {
    computedIndex = 0;
    nextindex = 500;
    nextindexExplorer = 10;
    ctx = canvas.getContext('2d');
    canvas.addEventListener("click", function (event) {
        var elem = new point(event.pageX, event.pageY - 90);
        elem.create();
    }, false);

    document.addEventListener('contextmenu', function (e) {

        for (var i = 0; i < points.length; i++) {
            if ((points[i].x + 5 > e.pageX && points[i].x - 10 < e.pageX) && ((points[i].y + 5 > e.pageY - 90 && points[i].y - 10 < e.pageY - 90))) {
                ctx.clearRect(points[i].x - 5 - 1, points[i].y - 5 - 1, 5 * 2 + 2, 5 * 2 + 2);
                if (i < computedIndex) {
                    computedIndex--;
                    deletedpoints.push(points[i]);
                }
                points.splice(i, 1);
                break;
            }
        }
        e.preventDefault();
    }, false);
    this.cmd("CreateLabel", 0, "", 20, 10, 0);
    this.animationManager.StartNewAnimation(this.commands);
    this.animationManager.skipForward();
    this.animationManager.clearHistory();

}

ThetaGraph.prototype.addControls = function () {
    this.controls = [];

    this.ha = addControlToAlgorithmBar("Button", "About Theta graph");
    this.ha.onclick = this.haCallback.bind(this);

    addLabelToAlgorithmBar("X: ");
    this.xPointField = addControlToAlgorithmBar("Text", "", "<div id='xtooltip'>0 < X < " + canvas.width + "</div>");
    this.xPointField.onkeypress = this.returnSubmit(this.xPointField,        // callback to make when return is pressed
                                               this.insert.bind(this),           // integer, max number of characters allowed in field
                                               4,                               // boolean, true of only digits can be entered.
                                               true);
    this.controls.push(this.xPointField);

    addLabelToAlgorithmBar("Y: ");
    this.yPointField = addControlToAlgorithmBar("Text", "", "<div id='ytooltip'>0 < Y < " + canvas.height + "</div>");
    this.yPointField.onkeypress = this.returnSubmit(this.yPointField,
                                               this.insert.bind(this),
                                               4,
                                               true);
    this.controls.push(this.yPointField);

    this.insertButton = addControlToAlgorithmBar("Button", "Insert");
    this.insertButton.onclick = this.insert.bind(this);
    this.controls.push(this.insertButton);

    addLabelToAlgorithmBar("k: ");
    this.kValue = addControlToAlgorithmBar("Text", "", "Please insert <strong>integer</strong> number k>3. It will be converted to t.<img src='img/theta-graph-k-t.png'>");



    addLabelToAlgorithmBar("OR  t: ");
    this.tValue = addControlToAlgorithmBar("Text", "", "Please insert <strong>real</strong> number t>1. It will be converted to k.<img src='img/theta-graph-t-k.png'>");
    this.tValue.onkeypress = this.returnSubmit(this.tValue,
                                               this.computeCallback.bind(this),
                                               8,
                                               true,
                                               this.kValue);
    this.kValue.onkeypress = this.returnSubmit(this.kValue,
                                               this.computeCallback.bind(this),
                                               3,
                                               true,
                                               this.tValue);
    this.controls.push(this.kValue);
    this.controls.push(this.tValue);

    this.computeButton = addControlToAlgorithmBar("Button", "Compute");
    this.computeButton.onclick = this.computeCallback.bind(this);
    this.controls.push(this.computeButton);
}

ThetaGraph.prototype.reset = function () {
    for (var i = 0; i < this.convexList.length; i++) {
        this.cmd("disconnect", this.convexList[i][0], this.convexList[i][1]);
    }
}
ThetaGraph.prototype.insert = function () {
    var xValue = parseInt(this.xPointField.value);
    var yValue = parseInt(this.yPointField.value);
    if (xValue != "" && yValue != "") {
        var elem = new point(xValue, yValue);
        elem.create();
        this.xPointField.value = "";
        this.yPointField.value = "";
    }
}

ThetaGraph.prototype.computeCallback = function () {
    if (parseInt("0" + this.kValue.value) < 4 && parseFloat("0" + this.tValue.value) <= 1) {
        alert("Please insert k or t value correctly");
        return;
    }
    this.implementAction(this.compute.bind(this), "");
}
ThetaGraph.prototype.haCallback = function () {
    window.open("AboutAlgorithm/AboutTheta_Graph.html", "MsgWindow", "width=740, height=720");

}
function sortByXValue(a, b) {
    return ((a.x < b.x) ? -1 : (a.x > b.x) ? 1 : 0);
}
function isRight(a, b, c) {
    return (((b.x - a.x) * (c.y - a.y) - (b.y - a.y) * (c.x - a.x)) > 0);
}
function getSpPoint(A, B, C) {
    var x1 = A.x, y1 = A.y, x2 = B.x, y2 = B.y, x3 = C.x, y3 = C.y;
    var px = x2 - x1, py = y2 - y1, dAB = px * px + py * py;
    var u = ((x3 - x1) * px + (y3 - y1) * py) / dAB;
    var x = x1 + u * px, y = y1 + u * py;
    return { x: x, y: y }; //this is D
}

function t2k(t) {
    var e1 = -1 * (Math.pow((1 / t), 2) - 1);
    var e2 = Math.asin(e1) / 2;
    var k = (2 * Math.PI) / e2;
    return Math.round(k).toString();

}

function k2t(k) {

    switch (k) {
        case 4:
        case 5:
        case 6:
        case 7:
        case 8:
            return "?";
        default:
            var e1 = (2 * Math.PI) / k;
            var t = 1 / (Math.cos(e1) - Math.sin(e1));
            return t.toFixed(2).toString();
    }

}

ThetaGraph.prototype.compute = function (pushedValue) {

    this.commands = [];
    this.reset();
    for (var l = 0; l < deletedpoints.length; l++) {
        this.cmd("delete", deletedpoints[l].objectId);
    }
    deletedpoints = [];

    for (var i = computedIndex; i < points.length; i++) {
        this.cmd("CREATEPOINT", points[i].objectId - 1, points[i].x, points[i].y, 1);
        this.cmd("SetForegroundColor", points[i].objectId - 1, ThetaGraph.PROJECT_COLOR);
        this.cmd("CREATEPOINT", points[i].objectId, points[i].x, points[i].y);
        this.cmd("SetBackgroundColor", points[i].objectId, ThetaGraph.DRAWPOINT_COLOR);
        this.cmd("Connect", points[i].objectId, points[i].objectId - 1, ThetaGraph.PROJECT_COLOR, 0.0, 0);
    }
    computedIndex = points.length;
    points.sort(sortByXValue);

    if (parseInt("0" + this.tValue.value) <= 0) {
        this.tValue.value = k2t(parseFloat(this.kValue.value));
    }
    else if (parseInt("0" + this.kValue.value) <= 0) {
        this.kValue.value = t2k(parseFloat(this.tValue.value));
    }
    var ret = this.makeExplorer();
    var step = ret[0];
    var startIndex = ret[1];
    var pointInExplorer = [];
    for (var i = 0; i < points.length; i++) {
        this.translateExplorer(points[i].x, points[i].y, step, pointInExplorer, startIndex);
        for (var j = 0; j < pointInExplorer.length - 2; j += 2) {
            var pointsInRegion = [];
            var pointsInRegionMoved = [];
            for (var k = 0; k < points.length; k++) {
                var bottom_status = isRight(points[i], pointInExplorer[j], points[k]);
                var top_status = isRight(points[i], pointInExplorer[j + 2], points[k]);
                if (bottom_status == false && top_status == true) {
                    var pcenter = getSpPoint(points[i], pointInExplorer[j + 1], points[k]);
                    pointsInRegion.push(points[k]);
                    pointsInRegionMoved.push(pcenter);
                    this.cmd("move", points[k].objectId - 1, pcenter.x, pcenter.y);
                }
            }
            if (pointsInRegion.length > 0) {
                this.cmd("Step");
                this.cmd("Step");

            }

            var mindist = Infinity;
            var minpoint = null;
            for (var k = 0; k < pointsInRegion.length; k++) {
                var a = points[i];
                var b = pointsInRegionMoved[k];
                var dist = Math.sqrt(Math.pow((b.x - a.x), 2) + Math.pow((b.y - a.y), 2));
                if (dist < mindist) {
                    mindist = dist;
                    minpoint = pointsInRegion[k];
                }
                this.cmd("move", pointsInRegion[k].objectId - 1, pointsInRegion[k].x, pointsInRegion[k].y);
            }
            if (minpoint != null) {
                this.cmd("SetEdgeHighlight", minpoint.objectId, minpoint.objectId - 1, 1);
                this.cmd("Step");

                this.cmd("SetHighlight", minpoint.objectId, 1);
                this.cmd("Step");

                this.cmd("disconnect", minpoint.objectId, points[i].objectId);
                this.cmd("Connect", points[i].objectId, minpoint.objectId, "#000000", 0.0, 0);
                this.convexList.push([points[i].objectId, minpoint.objectId]);
                this.cmd("Step");

                this.cmd("SetHighlight", minpoint.objectId, 0);
                this.cmd("SetEdgeHighlight", minpoint.objectId, minpoint.objectId - 1, 0);
                this.cmd("Step");
            }

        }
    }
    var indexer = startIndex;
    this.cmd("delete", indexer);
    for (var theta = 0; theta < 2 * Math.PI; theta += step) {
        indexer += 1;
        this.cmd("delete", indexer);
    }
    return this.commands;
}


ThetaGraph.prototype.makeExplorer = function () {
    var step = 2 * Math.PI / (parseInt(this.kValue.value) * 2);  // see note 1


    var indexer = nextindexExplorer;
    var startIndex = nextindexExplorer;
    this.cmd("CREATEPOINT", indexer, ThetaGraph.STARTPOINT_X, ThetaGraph.STARTPOINT_Y);
    for (var theta = 0; theta < 2 * Math.PI; theta += step) {
        //note 2.
        indexer += 1;
        var x = ThetaGraph.STARTPOINT_X + ThetaGraph.EPLORER_RADIOUS * Math.cos(theta);
        var y = ThetaGraph.STARTPOINT_Y - ThetaGraph.EPLORER_RADIOUS * Math.sin(theta);
        this.cmd("CREATEPOINT", indexer, x, y);
        if (indexer % 2 == 1) {
            this.cmd("Connect", startIndex, indexer, "#004400", 0.0, 0);
        }
        else {
            this.cmd("Connect", startIndex, indexer, "#B0FFA5", 0.0, 0);
        }

    }
    nextindexExplorer = indexer;
    this.cmd("Step");
    return [step, startIndex];


}

ThetaGraph.prototype.translateExplorer = function (x, y, step, pointInExplorer, startIndex) {
    var indexer = startIndex;
    this.cmd("move", indexer, x, y);
    console.log("Start index is :" + startIndex);
    for (var theta = 0; theta < 2 * Math.PI; theta += step) {
        //note 2.
        indexer += 1;
        var xt = x + ThetaGraph.EPLORER_RADIOUS * Math.cos(theta);
        var yt = y - ThetaGraph.EPLORER_RADIOUS * Math.sin(theta);
        this.cmd("move", indexer, xt, yt);
        pointInExplorer[indexer - startIndex - 1] = new point(xt, yt);
    }
    pointInExplorer[indexer - startIndex] = pointInExplorer[0];
    this.cmd("Step");

}


function point(xValue, yValue) {
    this.x = xValue;
    this.y = yValue;
    this.objectId = nextindex;
    this.create = function () {
        ctx.beginPath();
        ctx.arc(this.x, this.y, 5, 0, 2 * Math.PI, false);
        ctx.fillStyle = "#000000";
        ctx.fill();
        ctx.closePath();
        points.push(this);
        nextindex += 2;
    }
}


// Called by our superclass when we get an animation started event -- need to wait for the
// event to finish before we start doing anything
ThetaGraph.prototype.disableUI = function (event) {
    for (var i = 0; i < this.controls.length; i++) {
        this.controls[i].disabled = true;
    }
}

// Called by our superclass when we get an animation completed event -- we can
/// now interact again.
ThetaGraph.prototype.enableUI = function (event) {
    for (var i = 0; i < this.controls.length; i++) {
        this.controls[i].disabled = false;
    }
}

////////////////////////////////////////////////////////////
// Script to start up your function, called from the webapge:
////////////////////////////////////////////////////////////

var currentAlg;

function init() {
    var animManag = initCanvas();
    currentAlg = new ThetaGraph(animManag, canvas.width, canvas.height);

}


//function point(x1, y1) {
//    this.x = x1;
//    this.y = y1;
//}